@@ -1,5 +1,6 @@
 #' Quickly stops a parallel snowfall cluster and deregisters it from foreach.
 #' 
+#' @param kill Logical.  If TRUE, attempt to force-quit cluster if you get an out-of-control situation.  Default=FALSE.
 #' @param ... parameters to pass to sfStop()
 #' @author Jonathan A. Greenberg
 #' @details (Even more) quickly stop a snowfall cluster and sets foreach back
@@ -11,12 +12,21 @@
 #' @import foreach
 #' @export
 
-sfQuickStop <- function(...)
+sfQuickStop <- function(kill=FALSE,...)
 {
 	cl <- getOption("spatial.tools.current.cl")
 	registerDoSEQ()
-	stopCluster(cl)
-	options(spatial.tools.current.cl=NULL)
+	if(kill)
+	{
+		pid_connections <- lapply(getOption("spatial.tools.current.cl"),function(x) return(x$con))
+		sapply(pid_connections,function(x) close(x))
+		options(spatial.tools.current.cl=NULL)
+	} else
+	{
+		stopCluster(cl)
+		options(spatial.tools.current.cl=NULL)
+	}
+	
 }
 
 
